/****************************************************************************
 *
 *  ErrorDispatcher.h                        XMM Science Analysis Subsystem
 *
 *  Implementation of Error interface; handles errors.
 *
 *  (c) 1998  ESA  -  Written by Marco Beijersbergen
 *                    Registry inspired by Jon Brumfit
 *
 *  $Id: ErrorDispatcher.h,v 1.16 2002/06/27 12:31:31 jbakker Exp $
 *
 *****************************************************************************/

#ifndef ErrorDispatcher_h
#define ErrorDispatcher_h

#include "Error.h"
#include "SimpleHandler.h"
#include "SuppressHandler.h"

class ErrorDispatcher : public Error
{
  public:

    ErrorDispatcher();

    ~ErrorDispatcher();

    ErrorHandler* handler(ErrorHandler* handler);

    void name(const std::string& name) { _handler->name(name); }

    const std::string & name() const { return _handler->name(); }

    void verbosity(Verbosity level) { _handler->verbosity(level); }

    Verbosity verbosity() const { return _handler->verbosity(); }

    Verbosity verbosity(Layer layer) const { return _handler->verbosity(layer);}

    // Fatal error: this is so bad, nobody can continue
    void fatal(Code code);

    // Error: I can't continue, but my caller could
    void error(Code code);

    // Warning: report this message, but I will continue
    void warning(Code code);

    void message(Layer layer, Verbosity level);

    // Ignore warnings of type Code
    void ignoreWarning(Code code) { _suppressor.ignore(code);}

    // Allow warnings of type Code
    void allowWarning(Code code) { _suppressor.allow(code);}

    // Return the commandline switches
    std::string asCommandLine();

    std::ostream& errstream() { return _errstr; }

  private:

    // Member data
    std::ostringstream	_errstr;

    ErrorHandler*	_handler;

    SimpleHandler	_defaultHandler;
    SuppressHandler     _suppressor;
};


#endif
