// -*- C++ -*-
// ErrorHandler.h
//
// Abstract interface definition for error handling object.
//
// (c) 1997 European Space Agency - written by Marco Beijersbergen
//                                  inspired by Error written by Jon Brumfit

#ifndef ErrorHandler_h
#define ErrorHandler_h

#include <string>
#include <iostream>

#include "Msg.h"

//----------------------------------------------------------------
// Class ErrorHandler
//
// Abstract interface definition for error handling object
//----------------------------------------------------------------
extern "C" {
  typedef void (*ExternRoutine)();
}

class ErrorHandler : public Msg {

  public:

    typedef std::string Code;

    virtual ~ErrorHandler() {}

    virtual ErrorHandler& operator=(const ErrorHandler&) = 0;

    // Set the name of the client to be used in messages.
    virtual void name(const std::string&) = 0;

    // Returns the registered name.
    virtual const std::string& name() const = 0;

    // Set the global verbosity level.
    virtual void verbosity(Verbosity) = 0;

    // Returns the global verbosity level.
    virtual Verbosity verbosity() const = 0;

    // Returns the verbosity level of a particular layer.
    virtual Verbosity verbosity(Layer) const = 0;

    // Fatal error: this is so bad, nobody can continue
    virtual void fatal(Code code, const std::string& text ) = 0;

    // Error: I can't continue, but my caller could
    virtual void error(Code code, const std::string& text) = 0;

    // Warning: report this message, but I will continue
    virtual void warning(Code code, const std::string& text) = 0;

    // Message: report this message if the verbosity is higher than level
    virtual void message(Layer layer, Verbosity level, const std::string& text) = 0;

    // Possible trapping of non C++-code:
    //   extern "C" void foo_();
    //   tryExternal(foo_);
    virtual void tryExternal(ExternRoutine)=0;
};

#endif

