// -*-C++-*-
//
// File:      SimpleHandler.cc
// Author:    Jorgo Bakker (jbakker@rssd.esa.int)
// Generated: Mon Jan 27 13:48:09 CET 2003
// Usage:     -
// Info:      -

#include "SimpleHandler.h"

#include "AsciiFormatter.h"
#include "BinaryFormatter.h"

#include <sstream>
#include <cstdlib>
#include <csignal>
#include <Tracer.h>

SimpleHandler::SimpleHandler() : _formatter(0)
{
  const char* v = getenv("SAS_VERBOSITY");

  //gcc4.8.2 porting: initialization of _verbosity
  _verbosity = (v ?  atoi(v) :  0);

  // Separate channel?
  if(getenv("SAS_MESSAGE_CHANNEL")) {
    std::stringstream ss;
    ss << getenv("SAS_MESSAGE_CHANNEL");
    int channel=0;
    if (ss >> channel) {
      BinaryFormatter* b=new BinaryFormatter();
      b->setOutputChannel(channel);
      _formatter=b;
    }
  }

  if (!_formatter)  _formatter=new AsciiFormatter();
}

SimpleHandler::~SimpleHandler()
{
}

void SimpleHandler::fatal(Code code, const std::string& text) {
  _formatter->fatal(_name, code, text);
  stop();
} 

void SimpleHandler::error(Code code, const std::string& text)
{
  _formatter->error(_name, code, text);
  stop();
} 

void SimpleHandler::warning(Code code, const std::string& text) {
  _formatter->warning(_name, code, text);
} 

void SimpleHandler::message(Layer layer, Verbosity level, 
			    const std::string& text) {
  if (verbosity(layer) >= level)
    _formatter->message(_name, text);
}

void SimpleHandler::stop() {
  if(getenv("SAS_RAISE_SIGNAL")) // TODO: not used?
    raise(SIGUSR1);
  else
    exit(1);
}
